/* 
 *  GGI/DM blitting functions
 *
 *  Copyright (c) 1999 Nicolai Haehnle - Locinga@pfaffenwinkel.de
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#define DEBUG		1

#include <ggi/internal/dm_int.h>


/*
 * This function performs clipping at drawmap borders.
 *
 * ATTENTION: ggiCrossBlit has parameters the other way round (i.e source
 *	      first.
 * TODO: flags is not implemented
 */
int ggiDMBlit(ggidm_drawmap dest, int destx, int desty,
	      ggidm_drawmap src, int srcx, int srcy, int dx, int dy,
	      int flags)
{
	if (!dest || !src)
		return GGIDM_ERR_BADDRAWMAP;
	if (!(dest->flags & src->flags & GGIDM_ACQUIRED))
		return GGIDM_ERR_NOTACQUIRED;
	if (src->flags & GGIDM_INVALID)
		return GGIDM_ERR_INVALIDATED;

	dest->flags &= ~GGIDM_INVALID;

	/* perform simple clipping */
	if (destx < 0) {
		dx += destx;
		srcx -= destx;
		destx = 0;
	}
	if (desty < 0) {
		dy += desty;
		srcy -= desty;
		desty = 0;
	}
	if (srcx < 0) {
		dx += srcx;
		destx -= srcx;
		srcx = 0;
	}
	if (srcy < 0) {
		dy += srcy;
		desty -= srcy;
		srcy = 0;
	}
	
	if (destx + dx > dest->size.x) {
		dx = dest->size.x - destx;
	}
	if (desty + dy > dest->size.y) {
		dy = dest->size.y - desty;
	}
	if (srcx + dx > src->size.x) {
		dx = src->size.x - srcx;
	}
	if (srcy + dy > src->size.y) {
		dy = src->size.y - srcy;
	}

	if (destx > dest->size.x || desty > dest->size.y || 
	    srcx > src->size.x || srcy > src->size.y ||
	    dx <= 0 || dy <= 0)
		return 0;	/* would an error code be better? */

	return ggiCrossBlit(src->vis, src->pos.x+srcx, src->pos.y+srcy, dx, dy,
			    dest->vis, dest->pos.x+destx, dest->pos.y+desty);
}

/*
 * This function performs clipping at drawmap borders.
 *
 * ATTENTION: ggiCrossBlit has parameters the other way round (i.e source
 *	      first.
 * TODO: flags is not implemented
 */
int ggiDMBlitToVisual(ggi_visual_t dest, int destx, int desty,
		      ggidm_drawmap src, int srcx, int srcy, int dx, int dy,
		      int flags)
{
	if (!dest || !src)
		return GGIDM_ERR_BADDRAWMAP;
	if (!(src->flags & GGIDM_ACQUIRED))
		return GGIDM_ERR_NOTACQUIRED;
	if (src->flags & GGIDM_INVALID)
		return GGIDM_ERR_INVALIDATED;

	/* perform simple clipping */
	if (srcx < 0) {
		dx += srcx;
		destx -= srcx;
		srcx = 0;
	}
	if (srcy < 0) {
		dy += srcy;
		desty -= srcy;
		srcy = 0;
	}
	
	if (srcx + dx > src->size.x) {
		dx = src->size.x - srcx;
	}
	if (srcy + dy > src->size.y) {
		dy = src->size.y - srcy;
	}

	if (srcx > src->size.x || srcy > src->size.y || dx <= 0 || dy <= 0)
		return 0;	/* would an error code be better? */

	return ggiCrossBlit(src->vis, src->pos.x+srcx, src->pos.y+srcy, dx, dy,
			    dest, destx, desty);
}


/*
 * This function performs clipping at drawmap borders.
 *
 * ATTENTION: ggiCrossBlit has parameters the other way round (i.e source
 *	      first.
 * TODO: flags is not implemented
 */
int ggiDMBlitFromVisual(ggidm_drawmap dest, int destx, int desty,
			ggi_visual_t src, int srcx, int srcy, int dx, int dy,
			int flags)
{
	if (!dest || !src)
		return GGIDM_ERR_BADDRAWMAP;
	if (!(dest->flags & GGIDM_ACQUIRED))
		return GGIDM_ERR_NOTACQUIRED;

	dest->flags &= ~GGIDM_INVALID;

	/* perform simple clipping */
	if (destx < 0) {
		dx += destx;
		srcx -= destx;
		destx = 0;
	}
	if (desty < 0) {
		dy += desty;
		srcy -= desty;
		desty = 0;
	}
	
	if (destx + dx > dest->size.x) {
		dx = dest->size.x - destx;
	}
	if (desty + dy > dest->size.y) {
		dy = dest->size.y - desty;
	}

	if (destx > dest->size.x || desty > dest->size.y || 
	    dx <= 0 || dy <= 0)
		return 0;	/* would an error code be better? */

	GGIDPRINT("Do BlitFromVisual(dest: %p(%p), (%i/%i)/src: %p, (%i/%i))\n",
		dest, dest->vis, dest->pos.x+destx, dest->pos.y+desty,
		src, srcx, srcy);
	GGIDPRINT("Size: (%i/%i)\n", dx, dy);

	return ggiCrossBlit(src, srcx, srcy, dx, dy,
			    dest->vis, dest->pos.x+destx, dest->pos.y+desty);
}


int ggiDMPutBox(ggidm_drawmap dest, int destx, int desty,
	        void *src, int dx, int dy, int flags)
{
	if (!dest || !src)
		return GGIDM_ERR_BADDRAWMAP;
	if (!(dest->flags & GGIDM_ACQUIRED))
		return GGIDM_ERR_NOTACQUIRED;

	dest->flags &= ~GGIDM_INVALID;

	return ggiPutBox(dest->vis, dest->pos.x+destx, dest->pos.y+desty,
			 dx, dy, src);
}


int ggiDMGetBox(void *dest, ggidm_drawmap src, int srcx, int srcy, 
		int dx, int dy, int flags)
{
	if (!dest || !src)
		return GGIDM_ERR_BADDRAWMAP;
	if (!(src->flags & GGIDM_ACQUIRED))
		return GGIDM_ERR_NOTACQUIRED;
	if (src->flags & GGIDM_INVALID)
		return GGIDM_ERR_INVALIDATED;

	return ggiGetBox(src->vis, src->pos.x+srcx, src->pos.y+srcy, dx, dy,
			 dest);
}



